'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var react = require('react');
var classname = require('@bem-react/classname');
var classnames = require('@bem-react/classnames');

/* global Reflect, Promise */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function __spreadArray(to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
}

function withBemMod(blockName, mod, enhance) {
    var entity;
    var entityClassName;
    var modNames;
    function WithBemMod(WrappedComponent) {
        // Use cache to prevent create new component when props are changed.
        var ModifiedComponent;
        var modifierClassName;
        entity = entity || classname.cn(blockName);
        entityClassName = entityClassName || entity();
        var BemMod = react.forwardRef(function (props, ref) {
            modNames = modNames || Object.keys(mod);
            // TODO: For performance can rewrite `every` to `for (;;)`.
            var isModifierMatched = modNames.every(function (key) {
                var modValue = mod[key];
                var propValue = props[key];
                return modValue === propValue || (modValue === '*' && Boolean(propValue));
            });
            var nextProps = Object.assign({}, props, { ref: ref });
            if (isModifierMatched) {
                var modifiers = modNames.reduce(function (acc, key) {
                    if (mod[key] !== '*')
                        acc[key] = mod[key];
                    return acc;
                }, {});
                modifierClassName = modifierClassName || entity(modifiers);
                nextProps.className = classnames.classnames(modifierClassName, props.className)
                    // Replace first entityClassName for remove duplcates from className.
                    .replace("".concat(entityClassName, " "), '');
                if (typeof enhance === 'function') {
                    if (ModifiedComponent === undefined) {
                        ModifiedComponent = enhance(WrappedComponent);
                    }
                }
                else {
                    ModifiedComponent = WrappedComponent;
                }
                // Use createElement instead of jsx to avoid __assign from tslib.
                return react.createElement(ModifiedComponent, nextProps);
            }
            // Use createElement instead of jsx to avoid __assign from tslib.
            return react.createElement(WrappedComponent, nextProps);
        });
        return BemMod;
    }
    // Ignore `forwardRef` typings to keep compatibility with `HOC<T>`
    var withMod = WithBemMod;
    var _a = enhance || {}, _b = _a.__passToProps, __passToProps = _b === void 0 ? true : _b, _c = _a.__simple, __simple = _c === void 0 ? false : _c;
    var keys = Object.keys(mod);
    var isSimple = !enhance && keys.length === 1;
    var name = keys[0];
    var value = mod[keys[0]];
    withMod.__isSimple = isSimple || __simple;
    if (withMod.__isSimple) {
        withMod.__blockName = blockName;
        withMod.__mod = name;
        withMod.__value = value;
        withMod.__passToProps = __passToProps;
    }
    return withMod;
}
function createClassNameModifier(blockName, mod) {
    return withBemMod(blockName, mod, { __passToProps: false, __simple: true });
}
function composeSimple(mods) {
    var __blockName = mods[0].__blockName;
    var allMods = {};
    var allModsPassProps = {};
    var entity = classname.cn(__blockName);
    for (var _i = 0, mods_1 = mods; _i < mods_1.length; _i++) {
        var _a = mods_1[_i], __mod = _a.__mod, __value = _a.__value, __passToProps = _a.__passToProps;
        allMods[__mod] = allMods[__mod] || [];
        // для оптимизации поиска вводим простой массив вместо объекта
        allModsPassProps[__mod] = allModsPassProps[__mod] || [];
        allMods[__mod].push(__value);
        allModsPassProps[__mod].push(__passToProps);
    }
    var modNames = Object.keys(allMods);
    return function (Base) {
        function SimpleComposeWrapper(props) {
            var modifiers = {};
            var newProps = __assign({}, props);
            for (var _i = 0, modNames_1 = modNames; _i < modNames_1.length; _i++) {
                var key = modNames_1[_i];
                var modValues = allMods[key];
                var propValue = props[key];
                var foundModifierIndex = modValues.indexOf(propValue);
                // обрабатываем кейс когда у нас один модификатор со значением true
                if (foundModifierIndex === -1 && modValues[0] === true && propValue === false) {
                    foundModifierIndex = 0;
                }
                if (foundModifierIndex !== -1) {
                    modifiers[key] = propValue;
                    // если стоит флаг __passToProps = false, то не добавляем в пропсы
                    if (!allModsPassProps[key][foundModifierIndex]) {
                        delete newProps[key];
                    }
                    // если значение для модификатора undefined, то удаляем свойство
                }
                else if (newProps.hasOwnProperty(key) && propValue === undefined) {
                    delete newProps[key];
                }
            }
            newProps.className = entity(modifiers, [props.className]);
            return react.createElement(Base, newProps);
        }
        return SimpleComposeWrapper;
    };
}
/**
 * @param funcs higher order components
 *
 * @example
 * ```ts
 * import { compose } from '@bem-react/core';
 *
 * const Enhanced = compose(
 *     withBemMod('Component', { size: 's' }),
 *     withBemMod('Component', { theme: 'normal' }),
 * )(Component);
 * ```
 */
function compose() {
    // Use arguments instead of rest-arguments to get faster and more compact code.
    var fns = [].slice.call(arguments);
    var simple = [];
    var enhanced = [];
    for (var _i = 0, fns_1 = fns; _i < fns_1.length; _i++) {
        var f = fns_1[_i];
        f.__isSimple ? simple.push(f) : enhanced.push(f);
    }
    var oprimizedFns = simple.length ? __spreadArray([composeSimple(simple)], enhanced, true) : enhanced;
    return oprimizedFns.reduce(function (a, b) {
        return function () {
            return a(b.apply(0, arguments));
        };
    }, function (arg) { return arg; });
}
function composeU() {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    return compose.apply(void 0, args);
}

exports.compose = compose;
exports.composeU = composeU;
exports.createClassNameModifier = createClassNameModifier;
exports.withBemMod = withBemMod;
