import { ComponentType, FC } from 'react';
import { NoStrictEntityMods } from '@bem-react/classname';
export interface IClassNameProps {
    className?: string;
}
export type Enhance<T extends IClassNameProps> = (WrappedComponent: ComponentType<T>) => ComponentType<T>;
type withBemModOptions = {
    __passToProps: boolean;
    __simple: boolean;
};
export declare function withBemMod<T, U extends IClassNameProps = {}>(blockName: string, mod: NoStrictEntityMods, enhance?: Enhance<T & U> | withBemModOptions): {
    <K extends IClassNameProps = {}>(WrappedComponent: ComponentType<T & K>): (props: T & K) => React.ReactElement;
    __isSimple: boolean;
    __blockName: string;
    __mod: string;
    __value: string | number | boolean | undefined;
    __passToProps: boolean;
};
export declare function createClassNameModifier<T>(blockName: string, mod: NoStrictEntityMods): {
    <K extends IClassNameProps = {}>(WrappedComponent: ComponentType<T & K>): (props: T & K) => import("react").ReactElement<any, string | import("react").JSXElementConstructor<any>>;
    __isSimple: boolean;
    __blockName: string;
    __mod: string;
    __value: string | number | boolean | undefined;
    __passToProps: boolean;
};
export type ExtractProps<T> = T extends ComponentType<infer K> ? {
    [P in keyof K]: K[P];
} : never;
export type HOC<T> = (WrappedComponent: ComponentType) => ComponentType<T>;
export type Wrapper<T> = HOC<T>;
export type Composition<T> = <U extends ComponentType<any>>(fn: U) => FC<JSX.LibraryManagedAttributes<U, ExtractProps<U>> & T>;
export declare function compose<T1>(fn1: HOC<T1>): Composition<T1>;
export declare function compose<T1, T2>(fn1: HOC<T1>, fn2: HOC<T2>): Composition<T1 & T2>;
export declare function compose<T1, T2, T3>(fn1: HOC<T1>, fn2: HOC<T2>, fn3: HOC<T3>): Composition<T1 & T2 & T3>;
export declare function compose<T1, T2, T3, T4>(fn1: HOC<T1>, fn2: HOC<T2>, fn3: HOC<T3>, fn4: HOC<T4>): Composition<T1 & T2 & T3 & T4>;
export declare function compose<T1, T2, T3, T4, T5>(fn1: HOC<T1>, fn2: HOC<T2>, fn3: HOC<T3>, fn4: HOC<T4>, fn5: HOC<T5>): Composition<T1 & T2 & T3 & T4 & T5>;
export declare function compose<T1, T2, T3, T4, T5, T6>(fn1: HOC<T1>, fn2: HOC<T2>, fn3: HOC<T3>, fn4: HOC<T4>, fn5: HOC<T5>, fn6: HOC<T6>): Composition<T1 & T2 & T3 & T4 & T5 & T6>;
export declare function compose<T1, T2, T3, T4, T5, T6, T7>(fn1: HOC<T1>, fn2: HOC<T2>, fn3: HOC<T3>, fn4: HOC<T4>, fn5: HOC<T5>, fn6: HOC<T6>, fn7: HOC<T7>): Composition<T1 & T2 & T3 & T4 & T5 & T6 & T7>;
export declare function compose<T1, T2, T3, T4, T5, T6, T7, T8>(fn1: HOC<T1>, fn2: HOC<T2>, fn3: HOC<T3>, fn4: HOC<T4>, fn5: HOC<T5>, fn6: HOC<T6>, fn7: HOC<T7>, fn8: HOC<T8>): Composition<T1 & T2 & T3 & T4 & T5 & T6 & T7 & T8>;
export declare function compose(...fns: Array<HOC<any>>): Composition<any>;
type SafeUnionType<P, C> = P extends keyof C ? C[P] : never;
type DeepUnion2<T1, T2> = {
    [P in keyof (T1 & T2)]: SafeUnionType<P, T1> | SafeUnionType<P, T2>;
};
type DeepUnion3<T1, T2, T3> = {
    [P in keyof (T1 & T2 & T3)]: SafeUnionType<P, T1> | SafeUnionType<P, T2> | SafeUnionType<P, T3>;
};
type DeepUnion4<T1, T2, T3, T4> = {
    [P in keyof (T1 & T2 & T3 & T4)]: SafeUnionType<P, T1> | SafeUnionType<P, T2> | SafeUnionType<P, T3> | SafeUnionType<P, T4>;
};
type DeepUnion5<T1, T2, T3, T4, T5> = {
    [P in keyof (T1 & T2 & T3 & T4 & T5)]: SafeUnionType<P, T1> | SafeUnionType<P, T2> | SafeUnionType<P, T3> | SafeUnionType<P, T4> | SafeUnionType<P, T5>;
};
type DeepUnion6<T1, T2, T3, T4, T5, T6> = {
    [P in keyof (T1 & T2 & T3 & T4 & T5 & T6)]: SafeUnionType<P, T1> | SafeUnionType<P, T2> | SafeUnionType<P, T3> | SafeUnionType<P, T4> | SafeUnionType<P, T5> | SafeUnionType<P, T6>;
};
type DeepUnion7<T1, T2, T3, T4, T5, T6, T7> = {
    [P in keyof (T1 & T2 & T3 & T4 & T5 & T6 & T7)]: SafeUnionType<P, T1> | SafeUnionType<P, T2> | SafeUnionType<P, T3> | SafeUnionType<P, T4> | SafeUnionType<P, T5> | SafeUnionType<P, T6> | SafeUnionType<P, T7>;
};
type DeepUnion8<T1, T2, T3, T4, T5, T6, T7, T8> = {
    [P in keyof (T1 & T2 & T3 & T4 & T5 & T6 & T7 & T8)]: SafeUnionType<P, T1> | SafeUnionType<P, T2> | SafeUnionType<P, T3> | SafeUnionType<P, T4> | SafeUnionType<P, T5> | SafeUnionType<P, T6> | SafeUnionType<P, T7> | SafeUnionType<P, T8>;
};
export declare function composeU<T1>(fn1: HOC<T1>): Composition<T1>;
export declare function composeU<T1, T2>(fn1: HOC<T1>, fn2: HOC<T2>): Composition<DeepUnion2<T1, T2>>;
export declare function composeU<T1, T2, T3>(fn1: HOC<T1>, fn2: HOC<T2>, fn3: HOC<T3>): Composition<DeepUnion3<T1, T2, T3>>;
export declare function composeU<T1, T2, T3, T4>(fn1: HOC<T1>, fn2: HOC<T2>, fn3: HOC<T3>, fn4: HOC<T4>): Composition<DeepUnion4<T1, T2, T3, T4>>;
export declare function composeU<T1, T2, T3, T4, T5>(fn1: HOC<T1>, fn2: HOC<T2>, fn3: HOC<T3>, fn4: HOC<T4>, fn5: HOC<T5>): Composition<DeepUnion5<T1, T2, T3, T4, T5>>;
export declare function composeU<T1, T2, T3, T4, T5, T6>(fn1: HOC<T1>, fn2: HOC<T2>, fn3: HOC<T3>, fn4: HOC<T4>, fn5: HOC<T5>, fn6: HOC<T6>): Composition<DeepUnion6<T1, T2, T3, T4, T5, T6>>;
export declare function composeU<T1, T2, T3, T4, T5, T6, T7>(fn1: HOC<T1>, fn2: HOC<T2>, fn3: HOC<T3>, fn4: HOC<T4>, fn5: HOC<T5>, fn6: HOC<T6>, fn7: HOC<T7>): Composition<DeepUnion7<T1, T2, T3, T4, T5, T6, T7>>;
export declare function composeU<T1, T2, T3, T4, T5, T6, T7, T8>(fn1: HOC<T1>, fn2: HOC<T2>, fn3: HOC<T3>, fn4: HOC<T4>, fn5: HOC<T5>, fn6: HOC<T6>, fn7: HOC<T7>, fn8: HOC<T8>): Composition<DeepUnion8<T1, T2, T3, T4, T5, T6, T7, T8>>;
export declare function composeU(...fns: Array<HOC<any>>): Composition<any>;
export {};
